window.by = window.by || {};

// any client-side parts of the 'here is my availability' image + map

(function($, exports) {
    /**
     * this is async because it might take an AJAX call to get the user's hashid
     */
    exports.get_availability_html_async = function(options, callback, error_callback) {
        get_user_hashid_async(function(hashid) {
            callback(get_availability_html_from_hash(hashid, options));
        }, error_callback);
    };

    /**
     * Not particularly safe (hashID may not exist). Use carefully!
     */
    exports.get_availability_html = function(options) {
        return get_availability_html_from_hash(get_user_hashid(), options);
    };

    /**
     * Stores hashid as side effect but does not use it
     */
    exports.ensure_hashid_loaded = function(callbacks) {
        get_user_hashid_async(callbacks.success, callbacks.error);
    };

    /**
     * Possible options
     * - days_ahead: int
     * - start_date: javascript date object
     */
    var get_availability_html_from_hash = function(hashid, options) {
        // Part 1: building arguments for image //
        //////////////////////////////////////////
        var get = function(dict, key, otherwise) { return key in dict ? dict[key] : otherwise; };
        var recipient_can_propose_times_to_meet =
            get(options, 'include_clickable_times', true);
        var show_freebusy = get(options, 'show_freebusy', true);
        var tentativeId = get(options, 'tentativeId', "");
        var SKIP_WEEKENDS = '60';  // skip saturday and sunday
        var weekdays_to_skip = get(options, 'weekdays_to_skip', SKIP_WEEKENDS);
        // keep this in sync with calendar_backend/availability_map/views.py
        var IMAGECACHE_BUCKET_NAME = "bcal-calendar-imagecache";
        var S3_REGION = "s3-website-us-east-1";
        var location = get(options, 'location', "")


        // find start date (or leave empty for 'next few days, starting from today')
        var format_start_date_arg = function(date) {
            //yyyy-mm-dd
            var start_date_ymd = date.getFullYear() + "-" + (date.getMonth()+1) + "-" + date.getDate();
            return "&start_date_ymd=" + start_date_ymd;
        };

        var proposed_times_getarg = "";
        if (options.proposed_times) {
            // yyyy-mm-ddThh:mm:ss|more|events|like|that
            proposed_times_getarg = "&proposed_times=" +
                options.proposed_times.map(function(event) {
                    return "" + event.start.getTime()/1000
                         + "-to-"
                         + event.end.getTime()/1000;
                }).join("|");
        }


        var hour_first = 9;  // definitely show at least 9AM
        if (options.proposed_times) {
            hour_first = Math.min.apply(Math, options.proposed_times.map(
                function(event) { return event.start.getHours(); }
            ).concat(hour_first));
        }

        var hour_last = 20; // definitely show at least 8PM
        if (options.proposed_times) {
            hour_last = Math.max.apply(Math, options.proposed_times.map(
                function(event) { return event.end.getHours() + 1; }
            ).concat(hour_last));
        }

        var start_date_getarg = "";
        if (options.proposed_times) {
            var min_date_stamp = Math.min.apply(Math, options.proposed_times.map(
                function(event) { return new Date(event.start.getTime()).setHours(0,0,0,0); }
            ));
            options.start_date = new Date(min_date_stamp);
        }

        if (options.start_date) {
            start_date_getarg = format_start_date_arg(options.start_date);
        }

        var max_days_ahead = 5;
        if (options.days_ahead) {
            max_days_ahead = options.days_ahead
        } else if (options.proposed_times) {
            // enough days to show all the options
            var min_date_stamp = Math.min.apply(Math, options.proposed_times.map(
                function(event) { return new Date(event.start.getTime()).setHours(0,0,0,0); }
            ));
            var max_date_stamp = Math.max.apply(Math, options.proposed_times.map(
                function(event) { return new Date(event.end.getTime()).setHours(0,0,0,0); }
            ));
            max_days_ahead = Math.ceil((max_date_stamp - min_date_stamp) / 86400000) + 1;
        }

        // Part 2: preparing to build image //
        //////////////////////////////////////
        var IMAGE_WIDTH = 438;  // gmail mid width/arbitrary restriciton
        var HEIGHT_WHITESPACE_TOP = 16;   // enough space for alt-text on failed image load
        var HEIGHT_FIRSTROW_TOP = 16;
        var HEIGHT_TOP = HEIGHT_FIRSTROW_TOP + HEIGHT_WHITESPACE_TOP, HEIGHT_PER_HOUR = 42, HEIGHT_BOTTOM = 29;
        var X_BASE = 60, Y_BASE = HEIGHT_TOP, Y_INCR = HEIGHT_PER_HOUR / 2;
        var X_INCR = (IMAGE_WIDTH - X_BASE) / max_days_ahead;

        var image_height = HEIGHT_TOP + HEIGHT_PER_HOUR * (hour_last - hour_first) + HEIGHT_BOTTOM;

        var imagemap_name = "availability_map_" + Math.floor(new Date().getTime() / 1000);
        var image_id = "availability_map_img" + Math.floor(new Date().getTime() / 1000);

        var image_src_base = "https://boomerangcalendar.baydin.com";
        var image_src_args = "/availability/" + hashid
            + "?days_to_show=" + max_days_ahead
            + "&start_hour=" + hour_first
            + "&end_hour=" + hour_last
            + "&show_freebusy=" + show_freebusy
            + "&skipWeekdays=" + weekdays_to_skip
              // last two are optional
            + start_date_getarg
            + proposed_times_getarg;

        var image_src = image_src_base + image_src_args;

        // path for saved version, using url-safe chars
        var image_src_base64 = window.btoa(image_src_args).replace("+","-").replace("/", "_");
        var background_image_src = "http://" + IMAGECACHE_BUCKET_NAME + "." + S3_REGION + ".amazonaws.com/" + image_src_base64;

        var image_html = "<div><img "
            + "id='" + image_id + "' "
            + "alt='If you do not see the calendar, click Display Images above.' "
            + "src='" + image_src + "' "
            + "style='background: url(" + background_image_src + ")' "
            + "height='" + image_height + "' "
            + "width='" + IMAGE_WIDTH + "' "
            + "usemap='#" + imagemap_name
            + "'></div>";

        var recipients = window.by.ui_manips.extract_email_addresses_from_field('to');
        var recipient_getargs = "";
        if (recipients.length == 1) {
            // NOTE: we assume the recipient is in the 'to', and there is only one
            // otherwise we don't try to cleverly guess the recipient
            var recipient = recipients[0];
            recipient_getargs += "&recipient_email=" + recipient.email;
            if (recipient.name != "") {
                recipient_getargs += "&recipient_name=" + recipient.name;
            }
        }

        var subject_getarg = "";
        var subject = options.subject;
        if (subject != "") {
            subject_getarg = "&subject=" + encodeURIComponent(subject);
        }

        var tentative_getarg ="";
        if (tentativeId != "") {
            tentative_getarg = "&tentativeId=" + tentativeId;
        }


        var generate_clickable_area = function(options) {
            options.minutes_end = options.minutes_end || options.minutes_start + 30;
            options.explicitly_proposed = options.explicitly_proposed || false;
            // minutes are relative to hour_first,
            // so if hour_first = 9 and minutes = 30, that means 9:30.
            var coords = [X_BASE + X_INCR * options.days_ahead,
                          Y_BASE + Y_INCR * (options.minutes_start / 30),
                          X_BASE + X_INCR * (options.days_ahead + 1),
                          Y_BASE + Y_INCR * (options.minutes_end / 30)
            ];

            // 1:30 PM is 13 * 60 + 30 = 810
            var minutes_in_day = options.minutes_start + hour_first * 60;
            var minutes_pretty = String((Math.floor(minutes_in_day / 60) - 1) % 12 + 1)
                + (minutes_in_day % 60 == 30 ? ":30" : "")
                + (minutes_in_day >= 12 * 60 ? "PM" : "AM");
            var title = options.explicitly_proposed ?
                ('Confirm meeting at ' + minutes_pretty) :
                ('Propose meeting at ' + minutes_pretty);

            var url = 'https://boomerangcalendar.baydin.com/book_meeting/'
                + hashid
                + '?minutes=' + minutes_in_day
                + '&duration_minutes=' + (options.minutes_end - options.minutes_start)
                + '&daysahead=' + options.days_ahead
                + '&location=' + options.location
                + start_date_getarg
                + recipient_getargs
                + tentative_getarg
                + subject_getarg;
            return "<area target='_blank' href=\""+url+"\" shape='rect' coords='"+coords.join(',')+"' title='"+title+"'>";
        }

        var areas = [];
        if (options.proposed_times){
            var proposed_time_areas = options.proposed_times.map(function(event) {
            var days_ahead = days_between(options.start_date, event.start);
            var minutes_start = (event.start.getHours() - hour_first) * 60
                + event.start.getMinutes();
            var minutes_end = (event.end.getHours() - hour_first) * 60
                + event.end.getMinutes();

            return generate_clickable_area({
                days_ahead: days_ahead,
                minutes_start: minutes_start,
                minutes_end: minutes_end,
                explicitly_proposed: true,
                location: location
                });
            });
            areas = areas.concat(proposed_time_areas);
        }


        if (recipient_can_propose_times_to_meet) {
            for (var days_ahead = 0; days_ahead < max_days_ahead; days_ahead++) {
                for (var minutes = 0; minutes < (hour_last - hour_first) * 60; minutes+=30) {
                    areas.push(generate_clickable_area({
                        days_ahead: days_ahead,
                        minutes_start: minutes,
                        explicitly_proposed: false
                    }));
                }
            }
        }


        // "Powered by <a>Boomerang Calendar</a>"
        var bcal_banner_width = 140;
        var bcal_banner_coords = [IMAGE_WIDTH - bcal_banner_width,
                                  image_height - HEIGHT_BOTTOM,
                                  IMAGE_WIDTH,
                                  image_height
                                 ];
        areas.push("<area target='_blank' href='https://boomerangcalendar.com/?utm_source=shared_cal_imagemap' shape='rect' "
            + "coords='"+bcal_banner_coords.join(',')+"' title='Want to share your availability via email?  Install Boomerang Calendar'>");

        var map_html = "<map name='" + imagemap_name + "'>" + areas.join('') + "</map>";
        return image_html + map_html;
    };

    var get_hashid_key = function() {
        var HASH_VERSION = '2';
        return "bcal-user-hashid-version-" + HASH_VERSION + "-" + get_gmail_user();
    }

    /**
     * returns hashId or none. Kinda unsafe if key hasn't been loaded yet
     */
    var get_user_hashid = function() {
        return localStorage.getItem(get_hashid_key());
    }

    /**
     * Guarantees to return hashid properly.
     */
    var get_user_hashid_async = function(callback, error_callback) {
        callback = callback || function() {};
        error_callback = callback || function() {};  // make callbacks optional
        var hashid = get_user_hashid();
        if (hashid) {
            callback(hashid);
        } else {
            send_get_request_auth_wrapper(
                "https://boomerangcalendar.baydin.com/get_loggedinuser_hashid?guser=" + get_gmail_user(),
                function(data) {
                    var hashid = JSON.parse(decodeURIComponent(data)).hashid;
                    if (hashid) {
                        localStorage.setItem(get_hashid_key(), hashid);
                        callback(hashid);
                    } else {
                        error_callback("instead of hashid from server, we are getting " + data);
                        // TODO: log error to our server
                    }
                }
            );
        }
    }

})(jQuery, window.by.availability_map || (window.by.availability_map = {}));
