window.by = window.by || {};

(function($, exports) {
    var EMAIL_REGEX = /[a-zA-Z0-9\._+\-]+@[a-zA-Z0-9\.\-]+\.[a-z\.A-Z]+/g;
    var NO_ELEMS = { "plan-meetings": $(), "plan-group-meetings": $(), "next-few-days": $(), "next-week": $()};

    var MutationObserver = window.MutationObserver || window.WebKitMutationObserver || window.MozMutationObserver;

    // Pre-pop-in compose window: http://gmailblog.blogspot.co.uk/2012/10/introducing-new-compose-in-gmail.html
    var pre_pop_in_insert_links = (function() {
        function add_insert_meeting_links(){
            var frameDoc = get_frame();

            var $composeContainer = $(".M9 .fN form", frameDoc);

            if($composeContainer.length === 0) { return NO_ELEMS; } // Compose screen not done loading
            if($composeContainer.data("has-observer")) { return NO_ELEMS; }

            var $planMeeting = create_plan_meeting_link();
            var $planGroupMeeting = create_group_meeting_link_if_needed();

            new MutationObserver(function() {
                var $linkContainer = $(".cf .eF:has(.b2):visible", $composeContainer);

                if(!$linkContainer.is(":has(#b4cSuggestLink)")) {
                    $linkContainer.append($planMeeting.clone(true, true));
                }

                if(!$linkContainer.is(":has(#b4cGroupLink)")) {
                    $linkContainer.append($planGroupMeeting.clone(true, true));
                }
                $("#b4cGroupLink", $linkContainer).parent().toggle(show_group_meeeting_link());
            }).observe($composeContainer[0], { childList: true, subtree: true, characterData: true });

            $composeContainer.data("has-observer", true);
            $(".cf .eF:has(.b2)", $composeContainer)
                .append($planMeeting)
                .append($planGroupMeeting)
                ;

            return { "plan-meetings": $planMeeting, "plan-group-meetings": $planGroupMeeting, "next-few-days": $(), "next-week": $() };
        }

        function create_plan_meeting_link() {
            var b4cSuggestLink =
                "<span> <span class='eu' role='link' id='b4cSuggestLink' style='margin-top: -2px;'>" +
                "<img src='"+get_bcal_logo_url() +"' style='margin-bottom: -3px;'></img> Suggest Meeting" +
                "</span> &nbsp; </span>"
                ;

            return $(b4cSuggestLink)
                .find("img").css({ "position": "relative", "top": 1 }).end()
                .click(function() {
                    track_event("suggest-time-link", "click-compose-msg", (is_pop_out_window() ? "pop-out" : "main"));
                })
            .css("white-space", "nowrap")
                ;
        }

        function show_group_meeeting_link() {
            var frameDoc = get_frame();

            var recipients = $("[name=to]", frameDoc).first().val();
            var cc = $("[name=cc]", frameDoc).first().val();
            var emailRegEx = /[a-zA-Z0-9\._+\-]+@[a-zA-Z0-9\.\-]+\.[a-z\.A-Z]+/g;
            var numberRecipients = 0;
            var recipientMatch = (recipients + cc).match(emailRegEx);
            if (recipientMatch != null){
                numberRecipients = recipientMatch.length;
            }

            return numberRecipients > 1;
        }

        function create_group_meeting_link_if_needed() {
            var b4cSuggestLink =
                "<span><span class='eu' role='link' id='b4cGroupLink' style='margin-top: -2px;'>" +
                "<img src='" + get_bcal_logo_url() +"' style='margin-bottom: -3px;'></img> Plan Group Event" +
                "</span> &nbsp; </span>"
                ;

            track_event("plan-group-link", "appears", (is_pop_out_window() ? "pop-out" : "main"), null, true);

            return $(b4cSuggestLink)
                .hide()
                .find("img").css({ "position": "relative", "top": 1 }).end()
                .click(function() {
                    track_event("plan-group-link", "click-compose-msg", (is_pop_out_window() ? "pop-out" : "main"));
                })
            .css("white-space", "nowrap")
                ;
        }

        return add_insert_meeting_links;
    })();

    function default_insert_links() {
        var $gmailLinks =
            $(".pXSFsb.gU")
            .filter(function() { return $(this).find(".b4c-link").length === 0; });
        if ($gmailLinks.length == 0){
            $gmailLinks =
                $("[command=docs]").closest(".gU")
                .filter(function() { return $(this).find(".b4c-link").length === 0; });
        }
        if($gmailLinks.length > 0){ // after September 2, 2014
            var $template = $gmailLinks.find(".wG").first().clone().addClass("b4c-link");
            var $planMeetingTmpl = $template.clone();
            var $planNextFewDaysTmpl = $template.clone();
            var $planNextWeekTmpl = $template.clone();

            $planMeetingTmpl
                .addClass("b4c-plan-meeting")
                .attr("data-tooltip", "Suggest Meeting")
                .find(".a1").attr(
                    "style",
                    "background-image: url(" + get_insert_meeting_url() + ") !important;" +
                    "background-size: 20px !important;" + // background-size fixes retina displays
                    "background-position: 3px 1px !important;"
                )
            ;

            $planNextFewDaysTmpl
                .addClass("b4c-plan-group-meeting")
                .attr("data-tooltip", "Share Next Few Days")
                .find(".a1").attr(
                    "style",
                    "background-image: url(" + get_nextfewdays_availability_url() + ") !important;" +
                    "background-size: 20px !important;" + // background-size fixes retina displays
                    "background-position: 0px 0px !important;"
                )
            ;
            $planNextWeekTmpl
                .addClass("b4c-plan-group-meeting")
                .attr("data-tooltip", "Share Next Week's Availability")
                .find(".a1").attr(
                    "style",
                    "background-image: url(" + get_nextweek_availability_url() + ") !important;" +
                    "background-size: 20px !important;" + // background-size fixes retina displays
                    "background-position: 0px 0px !important;"
                )
            ;

            var $planMeetings = $(), $nextFewDays = $(), $nextWeek = $();
            $gmailLinks.each(function() {
                var $planMeetingLink = $planMeetingTmpl.clone(true, true);
                var $nextFewDaysLink = $planNextFewDaysTmpl.clone(true, true);
                var $nextWeekLink = $planNextWeekTmpl.clone(true, true);

                $planMeetings = $planMeetings.add($planMeetingLink);
                $nextFewDays = $nextFewDays.add($nextFewDaysLink);
                $nextWeek = $nextWeek.add($nextWeekLink);

                $planMeetings.click(function() {
                    track_event("suggest-time-link", "click-compose-msg", 'new-compose');
                });
                $nextFewDays.click(function() {
                    track_event("share-availability-click", "click-compose-msg", 'next-few-days');
                });
                $nextWeek.click(function() {
                    track_event("share-availability-click", "click-compose-msg", 'next-week');
                });

                var $this = $(this);
                // actually does the inserting
                var $linkContainer = $this.children("div").first();
                $linkContainer.append($planMeetingLink);
                $linkContainer.append($nextFewDaysLink)
                $linkContainer.append($nextWeekLink);

                // var $col = $this.closest("table").find("colgroup .Uy");
                // $col.before($col.clone()).before($col.clone()).before($col.clone());

                var $composeContainer = $this.closest(".I5");

                setTimeout(function() { // On reload, google is adjusting position with js, wait until their done.
                    $(".eq", $composeContainer) // Hidden icons container
                    .css("left", $this.position().left + 30); // reposition to compensate for new icons
                }, 0);
            });
        }
        else{ // before September 2, 2014. collapsible toolbar. Delete this branch when no longer needed
            $gmailLinks = $(".n1tfz .gU.er:not(.b4c-link)")
                .filter(function() { return $(this).siblings(".b4c-link").length === 0; });
            if($gmailLinks.length === 0)  { return NO_ELEMS; }

            var $template = $gmailLinks.first().clone().addClass("b4c-link");
            var $planMeetingTmpl = $template.clone();
            var $planNextFewDaysTmpl = $template.clone();
            var $planNextWeekTmpl = $template.clone();

            $planMeetingTmpl
                .addClass("b4c-plan-meeting")
                .find(".wG").attr("data-tooltip", "Suggest Meeting").end()
                .find(".a1").attr(
                    "style",
                    "background-image: url(" + get_insert_meeting_url() + ") !important;" +
                    "background-size: 20px !important;" + // background-size fixes retina displays
                    "background-position: 3px 1px !important;"
                )
            ;

            $planNextFewDaysTmpl
                .addClass("b4c-plan-group-meeting")
                .find(".wG").attr("data-tooltip", "Share Next Few Days").end()
                .find(".a1").attr(
                    "style",
                    "background-image: url(" + get_nextfewdays_availability_url() + ") !important;" +
                    "background-size: 20px !important;" + // background-size fixes retina displays
                    "background-position: 0px 0px !important;"
                )
            ;
            $planNextWeekTmpl
                .addClass("b4c-plan-group-meeting")
                .find(".wG").attr("data-tooltip", "Share Next Week's Availability").end()
                .find(".a1").attr(
                    "style",
                    "background-image: url(" + get_nextweek_availability_url() + ") !important;" +
                    "background-size: 20px !important;" + // background-size fixes retina displays
                    "background-position: 0px 0px !important;"
                )
            ;

            var $planMeetings = $(), $nextFewDays = $(), $nextWeek = $();
            $gmailLinks.each(function() {
                var $planMeetingLink = $planMeetingTmpl.clone(true, true);
                var $nextFewDaysLink = $planNextFewDaysTmpl.clone(true, true);
                var $nextWeekLink = $planNextWeekTmpl.clone(true, true);

                $planMeetings = $planMeetings.add($planMeetingLink);
                $nextFewDays = $nextFewDays.add($nextFewDaysLink);
                $nextWeek = $nextWeek.add($nextWeekLink);

                $planMeetings.click(function() {
                    track_event("suggest-time-link", "click-compose-msg", 'new-compose');
                });
                $nextFewDays.click(function() {
                    track_event("share-availability-click", "click-compose-msg", 'next-few-days');
                });
                $nextWeek.click(function() {
                    track_event("share-availability-click", "click-compose-msg", 'next-week');
                });

                var $this = $(this);
                // actually does the inserting
                $this.before($planMeetingLink).before($nextFewDaysLink).before($nextWeekLink);

                var $col = $this.closest("table").find("colgroup .Uy");
                $col.before($col.clone()).before($col.clone()).before($col.clone());

                var $composeContainer = $this.closest(".I5");

                setTimeout(function() { // On reload, google is adjusting position with js, wait until their done.
                    $(".eq", $composeContainer) // Hidden icons container
                    .css("left", $this.position().left + 30); // reposition to compensate for new icons
                }, 0);
            });
        }

        return { "plan-meetings": $planMeetings, "plan-group-meetings": $(), "next-few-days": $nextFewDays, "next-week": $nextWeek};
    }

    function add_insert_meeting_links() {
        if($(".M9 .fN form").length > 0) { return pre_pop_in_insert_links(); }

        return default_insert_links();
    }

    ////////////////////////////////////////////////////////////////////////////////////
    // Copied over from boomerang for Gmail:
    function toggle_boomerang_menu(e, owner, menuOptionClickCallback)
    {
        var frameDoc = get_frame();

        //We want to prevent clicks on the b4g menu from closing the menu
        var target = get_target(e);

        if(target.className.indexOf("J-Zh-I") === -1 && target.className.indexOf("f") !== 0 && target.className.indexOf("VP5otc-d2fWKd") === -1)
        {
            return;
        }

        var b4gMenu = frameDoc.getElementById("b4g_menu");
        if (b4gMenu)
        {
            close_b4g_menu(e, owner);
        }
        else
        {
            //Add "clicked" style to button
            $("div", owner).addClass("J-Zh-I-JO");
            $("div", owner).addClass("J-Zh-I-Kq");

            var b4gContainer = document.createElement("ul");
            b4gMenu = document.createElement("ul");

            b4gContainer.setAttribute("id","b4g_menu");
            b4gContainer.setAttribute("class","b4g_menu J-M");
            b4gContainer.setAttribute("style", "-webkit-user-select: none; min-width: 17em; min-height: auto; left:" + $(e.currentTarget).position().left + "px;");
            b4gContainer.setAttribute("role", "menu");
            b4gContainer.setAttribute("aria-haspopup", "true");
            b4gContainer.setAttribute("aria-activedescendant", "");

            b4gMenu = document.createElement("div");
            b4gMenu.setAttribute("id","b4g_inner_menu");
            b4gMenu.setAttribute("class","b4g_menu SK AX");
            b4gMenu.setAttribute("style","-webkit-user-select: none; font-size:12px;");

            b4gMenu.innerHTML = "<li class='menu-caption' style='list-style-type:none;margin-left:0;padding: 2px 2.5px 2px 6px;color:gray; padding-bottom:5px;'>" +
                "Return Conversation to Inbox: " +
                "<img class='b4g_menu' id='notes_toggle' src='" + B4G_STATIC_FILES_DIRECTORY + "notes_off.png' style='cursor:pointer;' title='Click to enter a note for this message'/>" + "</li>"
            ;


            var menuItem = document.createElement("div");
            $(menuItem).append(
                "<h3 class='b4g_menu'>Enter your note below.</h3>" +
                "<textarea cols='25' rows='15' id='note_entry' class='b4g_menu' style='border: 1px solid #999; background-color:#fced78;width:100%;box-sizing:border-box;'>Some text</textarea>" +
                "<div class='b4g_menu'>This note will return with your message.<br/></div>" +
                "<button class='b4g_menu' id='notes_save_button' style='margin: 15px 20px 0 90px;'>Save</button>"+
                "<span class='b4g_menu' id='notes_cancel_button' style='text-decoration:underline; cursor:pointer; color:#15C'>Cancel</span>"
            );
            menuItem.setAttribute("class", "b4g_menu SK AX" );
            menuItem.setAttribute("id", "notes_menu" );
            menuItem.setAttribute("style", "display:none; min-width: 17em; min-height: auto; left:2px; margin:5px" );
            b4gMenu.appendChild(menuItem);


            menuItem = document.createElement("div");
            var condBoomCheckBox = document.createElement("input");
            condBoomCheckBox.setAttribute("style", "margin: 0 3px 5px 3px; border-bottom:1px solid #dcdcdc;" );
            condBoomCheckBox.type = "checkbox";
            condBoomCheckBox.name = "condBoomerang";
            condBoomCheckBox.id = "conditionalBoomerang";
            condBoomCheckBox.value = false;
            condBoomCheckBox.setAttribute("class", "b4g_menu" );
            $(condBoomCheckBox).change(function() {
                track_event([ "thread menu", "conditional boomerang", ($(this).is(":checked") ? "checked" : "un-checked") ]);
            });

            menuItem.appendChild(condBoomCheckBox);
            $(menuItem).append("Only if nobody responds");
            menuItem.setAttribute("class", "b4g_menu" );
            b4gMenu.appendChild(menuItem);

            b4gMenu.appendChild(build_b4g_menu_item("In 1 hour", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_menu_item("In 2 hours", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_menu_item("In 4 hours", menuOptionClickCallback, true));
            b4gMenu.appendChild(build_b4g_menu_item("Tomorrow morning", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_menu_item("Tomorrow afternoon", menuOptionClickCallback, true));
            b4gMenu.appendChild(build_b4g_menu_item("In 2 days", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_menu_item("In 4 days", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_menu_item("In 1 week", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_menu_item("In 2 weeks", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_menu_item("In 1 month", menuOptionClickCallback, true));

            var specificTimeHeader = document.createElement("li");
            specificTimeHeader.setAttribute("class","menu-caption");
            specificTimeHeader.innerHTML="At a specific time";
            specificTimeHeader.setAttribute("style", "margin-left:3px");
            b4gMenu.appendChild(specificTimeHeader);


            menuItem = document.createElement("li");
            menuItem.setAttribute("style", "color: #333; font-style: italic; font-weight: bold; font-size: 0.8em; margin-top: 3px; margin-left: 3px;");
            menuItem.setAttribute("class", "b4g_menu");
            menuItem.innerHTML = "Examples: 'Monday 9am', 'Dec 23'<br/>";
            b4gMenu.appendChild(menuItem);
            var formItem = document.createElement("form");
            formItem.setAttribute("onsubmit", "return false;");
            menuItem = document.createElement("div");
            menuItem.setAttribute("style","padding-top:3px; vertical-align: middle;");
            menuItem.setAttribute("class", "b4g_menu");
            if(last_date_selected == null || last_date_selected < new Date())
            {
                the_date = Date.today().addDays(2).addHours(8);
            }
            else
            {
                the_date = last_date_selected;
            }
            var strDate = the_date.toString("M/d/yyyy h:mm tt");
            var inputBox = document.createElement("input");
            inputBox.setAttribute("style", "margin-left: 3px;");
            inputBox.setAttribute("class", "b4g_menu");
            inputBox.type = "text";
            inputBox.name = "time";
            inputBox.id = "datepicker";

            inputBox.value = strDate;
            inputBox.addEventListener('keyup', update_date_preview , false);
            inputBox.addEventListener('focus', update_date_preview , false);
            menuItem.appendChild(inputBox);
            formItem.appendChild(menuItem);

            menuItem = document.createElement("div");
            menuItem.setAttribute("style", "color:#969696; margin-left: 3px;");
            menuItem.innerHTML = "<span id='date-preview' class='b4g_menu' style='color:#327a01;'></span>";
            formItem.appendChild(menuItem);

            menuItem = document.createElement("div");
            menuItem.setAttribute("class", "b4g_menu");
            var button = document.createElement("input");
            button.type = "submit";
            button.value = "Confirm";
            button.setAttribute("style", "margin: 3px 0 5px 3px; ");
            button.setAttribute("class", "b4g_menu");
            button.addEventListener('click', function(e) {
                specific_time_chosen(e, menuOptionClickCallback);
            }, false);
            menuItem.appendChild(button);
            formItem.appendChild(menuItem);
            b4gMenu.appendChild(formItem);

            // Add customized times
            var customTimes = load_three_most_frequent_dates();
            for (var idx = 0; idx<customTimes.length; idx++)
            {
                menuItem = build_custom_b4g_menu_item(customTimes[idx], menuOptionClickCallback);
                b4gMenu.appendChild(menuItem);
            }
            menuItem.setAttribute("style", "border-bottom: 1px solid #dcdcdc; padding-bottom:5px; margin-bottom:5px;");
            menuItem.setAttribute("class", "b4g_menu");

            var randTime = document.createElement("li");
            randTime.innerHTML = "<a id='random-time' style='border-width:0px;color:black;padding: 2px 2.5px 2px 6px;cursor:pointer;display:block;margin-bottom:5px;' class='menu-anchor'><img src='" + B4G_STATIC_FILES_DIRECTORY + "tree-plus.png'></img>&nbsp;Random time</a>";
            $(randTime.firstChild).hover(
                function() { $(this).addClass("J-N-JT"); },
                function() { $(this).removeClass("J-N-JT"); }
            );

            var rOpen = function()
            {
                random_time_open(this, b4gMenu);
                adjust_b4g_menu_height();

                track_event([ "thread menu", "random time", "opened" ]);
            };
            var rClose = function()
            {
                random_time_close(this, b4gMenu);
                adjust_b4g_menu_height();

                track_event([ "thread menu", "random time", "closed" ]);
            };
            $(randTime).toggle(rOpen, rClose);
            b4gMenu.appendChild(randTime);

            b4gMenu.appendChild(build_b4g_random_menu_item("- By 5pm today", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_random_menu_item("- Within 1 week", menuOptionClickCallback, false));
            b4gMenu.appendChild(build_b4g_random_menu_item("- Within 1 month", menuOptionClickCallback, true));

            b4gContainer.appendChild(b4gMenu);
            $(owner).parent().append(b4gContainer);

            // External clicks will close the Boomerang menu.
            $(frameDoc).bind('click', {owner: owner}, b4g_menu_frameDoc_click_handler);

            bind_datepicker(inputBox);

            // Prevent Gmail from making the textbox unselectable
            $(inputBox, frameDoc).mousedown(function(event){
                event.stopImmediatePropagation();
            });
            $(inputBox, frameDoc).css("-moz-user-select","text");

            // Focus input box for immediate typing
            inputBox.focus();
            inputBox.select();

            $('span#date-preview.b4g_menu',frameDoc).text("");

            // Event handlers for notes features
            $("#note_entry", frameDoc).val("");

            $("#notes_toggle", frameDoc).click(function(){
                send_image_request("https://b4g.baydin.com/mailcruncher/checkaccountstatus?guser=" + encodeURIComponent(get_gmail_user()),notes_callback, "");
            });

            $("#notes_save_button", frameDoc).click(function(){
                toggle_b4g_notes();
                if ($("#note_entry", frameDoc).val()){
                    $("#notes_toggle", frameDoc).attr("src", B4G_STATIC_FILES_DIRECTORY + 'notes_on.png');
                }
                adjust_b4g_menu_height();
            });
            $("#notes_cancel_button", frameDoc).click(function(){
                toggle_b4g_notes();
                $("#note_entry", frameDoc).val("");
                $("#notes_toggle", frameDoc).attr("src", B4G_STATIC_FILES_DIRECTORY + 'notes_off.png');
                adjust_b4g_menu_height();
            });

            adjust_b4g_menu_height();
        }
    }

    function get_compose_container(elem) {
        var $container, $elem = $(elem);

        $container = $elem.closest(".nH.Hd[role='dialog']"); // Should catch normal pop-in compose
        if($container.length > 0)  { return $container; }

        $container = $elem.closest(".aoI"); // Catches replies in post pop-in compose
        if($container.length > 0) { return $container; }

        $container = $elem.closest(".ip.adB"); // Catches replies in pre pop-in compose
        if($container.length > 0) { return $container; }

        $container = $elem.find(".aoI"); // Popout compose and popout replies
        if($container.length > 0) { return $container; }

        console.log ( "failed to find compose");
        return document; // Last ditch effort.  Something probably went wrong.
    }

    function get_compose_text_body($composeContainer) {
        // For pre-pop-in compose: ".Am.Al:not(:has(iframe))" for HTML compose, plain text is handled later
        // For new pop-in compose: ".LW-avf" for chrome and ".Am.Al > iframe" for firefox
        var $composeTextBody = $(".Am.Al:not(:has(iframe)), .LW-avf, .Am.Al > iframe", $composeContainer);

        // If neither selector worked, that means composeContainer isn't done "loading".
        // Instead of making this function async, gonna leave it up to callers to deal with it.
        if(!$composeTextBody[0]) { return null; }

        // In fireox and pre-pop-in text body is an iframe:
        if($composeTextBody[0].contentDocument) { $composeTextBody = $($composeTextBody[0].contentDocument.body); }

        return $composeTextBody;
    }
    //end copy
    ////////////////////////////////////////////////////////////////////////////////////

    function extract_subject_from_composer(frameDoc) {
        if (!frameDoc){
            frameDoc = get_frame();
        }

        return $("[name='subject']", frameDoc).val();
    }

    function extract_email_addresses_from_field(fieldName, frameDoc)
    {
      if (!frameDoc){
        frameDoc = get_frame();
      }
      var fields = $("[name='"+ fieldName +"']", frameDoc);
      var results = [];
      fields.each(function(){
        var field_itemized = $(this).val().split(" ");
        if (field_itemized != "") {
          var email_with_lt_gt = field_itemized[field_itemized.length - 1];
          var email_address_matches = email_with_lt_gt.match(EMAIL_REGEX);
          var email_address;
          if (email_address_matches) {
            email_address = email_address_matches[0];
          }
          else {
            email_address = "";
          }

          results.push({
            email: email_address,
            name: field_itemized.slice(0, field_itemized.length - 1).join(" ")
          });
        }
      });

      return results;
    }

    var observe_composer_text = function(observe_func, context_element, config) {
        // copying default composer from example
        var context_element = context_element || document;
        var wait_for_compose_to_load = function() {
            var composer_text = get_compose_text_body(get_compose_container(context_element));
            if (!composer_text) {
                // composer in load state, let's keep waiting
                // maybe switch this to another MutationObserver
                setTimeout(wait_for_compose_to_load, 500);
            } else {
                var config = config || {
                    characterData: true,
                    subtree: true
                };
                new MutationObserver(function(mutations) {
                    var observer = this;
                    for(var i = 0; i < mutations.length; i++) {
                        var mutation = mutations[i];
                        var result = observe_func(mutation.target.data);
                        if (result == "done") {
                            observer.disconnect();
                            break;
                        }
                    };
                }).observe(composer_text[0], config);
            }
        }
        wait_for_compose_to_load();
    }

    exports.add_insert_meeting_links = add_insert_meeting_links;
    exports.observe_composer_text = observe_composer_text;
    exports.extract_email_addresses_from_field = extract_email_addresses_from_field;
    exports.extract_subject_from_composer = extract_subject_from_composer;
})(jQuery, window.by.ui_manips || (window.by.ui_manips = {}));
