window.by = window.by || {};

(function($, exports) {
	var menuTemplate =
		'<ul class="by-timezone-menu">' +
			'<li><input class="timezone-search" type="search" placeholder="Find more..."/></li>' +
			'<li><a class="timezone-option" data-timezone-offset="30" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="-30" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="60" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="-60" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="90" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="-90" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="120" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="-120" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="150" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="-150" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="180" href="#"></a></li>' +
			'<li><a class="timezone-option" data-timezone-offset="-180" href="#"></a></li>' +
		'</ul>'
	;
	var timezoneHeader =
		'<a class="by-spawns-menu" href="#">' +
			'<span class="timezone-abbr"></span>' +
			'<span class="by-icon by-icon-triangle-1-s"></span>' +
		'</a>'
	;

	var timezones = [], offsetToConical = {};

	var urls = get_timezone_data_urls();

	$.get(urls.offsets, function(data) { offsetToConical = data; }, "json");
	$.get(urls.info, function(data) { timezones = data; }, "json");

	function prettyPrintTimezone(info) {
		var showNegative = info["minute-offset"] > 0; // The offset the browser uses is backwards
		var hours = Math.abs(info["minute-offset"] / 60) >> 0;
		var minutes = Math.abs(info["minute-offset"] % 60);

		var GMT = "GMT " + (showNegative ? "-" : "+") + (hours < 10 ? "0" : "") +
			hours + ":" + (minutes < 10 ? "0" : "") + minutes;

		return GMT + " " + info.name + " (" + info.abbreviation + ")";
	}

	function hideMenus() {
		$(".by-timezone-menu")
			.add($removeTimezone)
			.add($addSubMenuTimezone)
			.add($addTimezone)
			.hide()
			.find(".timezone-search").val("");
	}

	function updateMenu($menu, timezone, callback) {
		if(!isFinite(timezone)) { return; }

		$menu.find(".timezone-option").each(function() {
			var info = offsetToConical[timezone + parseInt($(this).data("timezone-offset"), 10)];

			$(this).toggle(!!info);
			if(!info) { return; }

			$(this)
				.data("timezone", info)
				.text(prettyPrintTimezone(info));
		});
		if(callback) { callback(null, offsetToConical[timezone]); }
	}

	function autocompleteTimezoneSource(req, res) {
		var term = req.term.toLowerCase();

		var data = [];
		for(var i = 0; i < timezones.length; i++) {
			var info = timezones[i];

			if(info.abbreviation.toLowerCase().indexOf(term) < 0 &&
			info.name.toLowerCase().indexOf(term) < 0) { continue; }

			data.push({ label: prettyPrintTimezone(info), info: info });
		}

		res(data);
	}

	function createMenu($menu, callback) {
		if(Object.prototype.toString.call($menu) === "[object Function]") {
			callback = $menu;
			$menu = null;
		}

		$menu = $menu || $(menuTemplate);

		$menu
			.menu()
			.css({ position: "absolute", "z-index": 10000 })
			.hide()
			.bind("by-timezone-change", function(e, info) {
				callback(null, info);
				$(this).hide();
			})
			.delegate(".timezone-option", "click", function(e) {
				e.preventDefault();

				$menu.trigger("by-timezone-change", [ $(this).data("timezone") ]);
			})
			.find(".timezone-search").autocomplete({
				minLength: 2,
				select: function(e, ui) {
					e.preventDefault();

					$menu.trigger("by-timezone-change", [ ui.item.info ]);
				},
				source: autocompleteTimezoneSource
			})
			.end()
		;

		return $menu;
	}

	var $addTimezone = $(menuTemplate)
		.prepend("<li class='menu-label'>Switch to</li>")
		.prepend("<li class='by-menu-separator'></li>")
		.prepend("<li><a class='sub-menu-opener add-timezone' href='#'>Add secondary timezone <span class='by-icon by-icon-triangle-1-e'></span></a></li>")
	;
	var $removeTimezone = $(menuTemplate)
		.prepend("<li class='menu-label'>Switch to</li>")
		.prepend("<li class='by-menu-separator'></li>")
		.prepend("<li><a class='remove-timezone' href='#'>Remove secondary timezone</a></li>")
	;
	var $addSubMenuTimezone = $(menuTemplate)
		.prepend("<li class='menu-label'>Select second timezone</li>")
	;

	var $tzPrimary = $(timezoneHeader)
		.find(".timezone-abbr")
			.addClass("primary-timezone-header")
			.text("???")
		.end()
		.click(function(e) {
			e.preventDefault();
			$(".by-menu").not($addTimezone).hide();
			$addTimezone.show().position({
				of: this,
				my: "left top",
				at: "left bottom"
			}).find(".timezone-search").focus();
		})
	;
	var $tzSecondary = $(timezoneHeader)
		.find(".timezone-abbr")
			.addClass("secondary-timezone-header")
			.text("???")
		.end()
		.click(function(e) {
			e.preventDefault();
			$(".by-menu").not($removeTimezone).hide();
			$removeTimezone.show().position({
				of: this,
				my: "left top",
				at: "left bottom"
			}).find(".timezone-search").focus();
		})
	;

	var initMenus = function() {
		initMenus = $.noop; // To ensure menus only init onces

		createMenu($addTimezone, function(err, offset) {
			if(err) { throw new Error("Error creating add-timezone menu."); }

			var $calendar = $addTimezone.closest(".by-menu").data("fullCalendar");
			$calendar.fullCalendar("option", "primary-agenda-timezone", offset["minute-offset"]);
		}).appendTo(document.body);
		createMenu($removeTimezone, function(err, offset) {
			if(err) { throw new Error("Error creating remove menu."); }

			var $calendar = $removeTimezone.closest(".by-menu").data("fullCalendar");
			$calendar.fullCalendar("option", "secondary-agenda-timezone", offset["minute-offset"]);
		}).appendTo(document.body);
		createMenu($addSubMenuTimezone, function(err, offset) {
			if(err) { throw new Error("Error creating sub menu."); }

			var $calendar = $addSubMenuTimezone.closest(".by-menu").data("fullCalendar");
			$calendar.fullCalendar("option", "secondary-agenda-timezone", offset["minute-offset"]);
		}).appendTo(document.body);

		$addTimezone.find(".sub-menu-opener")
			.click(function(e) {
				e.preventDefault();
				$addSubMenuTimezone.show().position({
					of: this,
					my: "left top",
					at: "right top",
					offset: "1px -3px"
				}).find(".timezone-search").focus();
			})
		;

		$removeTimezone.find(".remove-timezone")
			.click(function(e) {
				e.preventDefault();

				var $calendar = $(this).closest(".by-menu").data("fullCalendar");
				$calendar.fullCalendar("option", "secondary-agenda-timezone", Number.NaN);
			})
		;

		$(document).add(get_frame()).click(function(e) {
			var $target = $(e.target);

			if($target.closest(".by-timezone-menu").length > 0) { return; }

			if($target.closest(".by-spawns-menu").length > 0) { return; }

			hideMenus();
		});
	};

	function addMenusToCalendar(calendar) {
		var $calendar = $(calendar);

		initMenus();

		$addTimezone.add($removeTimezone).add($addSubMenuTimezone)
			.data("fullCalendar", $calendar)
		;

		$(".timezone-abbr", $tzPrimary)
			.text(offsetToConical[$calendar.fullCalendar("option", "primary-agenda-timezone")].abbreviation)
		;

		updateMenu(
			$addTimezone,
			$calendar.fullCalendar("option", "primary-agenda-timezone")
		);
		updateMenu(
			$addSubMenuTimezone,
			$calendar.fullCalendar("option", "primary-agenda-timezone")
		);
		updateMenu(
			$removeTimezone,
			$calendar.fullCalendar("option", "primary-agenda-timezone")
		);

		$calendar
			.unbind("primary-agenda-timezone-change secondary-agenda-timezone-change")
			.bind("primary-agenda-timezone-change secondary-agenda-timezone-change", function(e, value) {
				hideMenus();

				var headers = $calendar.fullCalendar("option", "timezone-headers").agendaWeek;
				var isPrimary = (e.type === "primary-agenda-timezone-change");
				var offset = offsetToConical[value] || { abbreviation: "???" };
				(isPrimary ? headers.primary : headers.secondary)
					.find(".timezone-abbr").text(offset.abbreviation).end()
					.data("timezone-offset", value);

				if(isPrimary) {	updateMenu($addTimezone, value); }
				else {
					updateMenu($addSubMenuTimezone, value);
					updateMenu($removeTimezone, value);
				}
			})
			.fullCalendar(
				"option",
				"timezone-headers",
				{ primary: $tzPrimary.clone(true, true), secondary: $tzSecondary.clone(true, true) }
			)
			.parents(":ui-dialog").bind("dialogclose", function() { hideMenus(); })
		;
	}

	exports.createMenu = createMenu;
	exports.updateMenu = updateMenu;
	exports.addMenusToCalendar = addMenusToCalendar;
})(jQuery, window.by.timezone || (window.by.timezone = {}));
