var LOCAL_STORAGE_LAST_UPDATED = "bcal-settings-last-updated-";
var LOCAL_STORAGE_EVENTS = "bcal-events-";
var LOCAL_STORAGE_LOOKUP = "bcal-event-lookup-";
var LOCAL_STORAGE_CACHE_MAXIMUM_DATE_VALID = "bcal-maximum-cache-date-";
var LOCAL_STORAGE_BLOCK_THIS_ACCOUNT = "bcal-block-";
var LOCAL_STORAGE_PREFERENCES = "bcal-preferences-";
var LOCAL_STORAGE_TIME_ZONE = "bcal-time-zone-";

var MINUTES_TO_CHECK_IF_LOCAL_STORAGE_IS_IN_SYNC = 10;
var LOCAL_STORAGE_UPDATE_FREQUENCY_MINUTES = 60;

var DEFAULT_GET_EVENTS_MIN = -30;
var DEFAULT_GET_EVENTS_MAX = 400;


function get_google_apps_sync_count_within_a_day(){
	var googleAppsCount = 0;
	for (var key in localStorage){
		if (key.indexOf("bcal-settings-last-updated-") === 0){
			var isGoogleApps = key.indexOf("@gmail.com") === -1;
			
			var lastUpdatedDateStr = localStorage.getItem(key);
			var lastUpdatedDate = Date.parse(lastUpdatedDateStr);
			
			if (isGoogleApps && lastUpdatedDate !== null && lastUpdatedDate > new Date().addDays(-1))
			{
				googleAppsCount++;
			}
		}
	}
	return googleAppsCount;
}


function sync_with_google_calendar_if_necessary()
{
	var lastUpdatedDateStr = read_from_localStorage(LOCAL_STORAGE_LAST_UPDATED);
	var maximumCacheDateValidStr = read_from_localStorage(LOCAL_STORAGE_CACHE_MAXIMUM_DATE_VALID);
	var events = read_from_localStorage(LOCAL_STORAGE_EVENTS);
	var lookup = read_from_localStorage(LOCAL_STORAGE_LOOKUP);
	var timeZone = read_from_localStorage(LOCAL_STORAGE_TIME_ZONE);

	var lastUpdatedDate;
	var maximumCacheDateValid;
	if (lastUpdatedDateStr != null &&  maximumCacheDateValidStr!= null)
	{
		lastUpdatedDate = Date.parse(lastUpdatedDateStr);
		maximumCacheDateValid = Date.parse(maximumCacheDateValidStr);
	}

	var hasNoCache = lastUpdatedDateStr == null || maximumCacheDateValidStr == null || events == null || lookup == null;
	var cacheIsNoLongerFresh = maximumCacheDateValid == null || maximumCacheDateValid < new Date().addDays(DEFAULT_GET_EVENTS_MAX - 100);
	var timeZoneOutOfSync = timeZone == null || timeZone !== get_olson_database_timezone_name();

	if (!is_pop_out_window())
	{
		if (hasNoCache || cacheIsNoLongerFresh || timeZoneOutOfSync){
			sync_with_google_calendar_helper();
		}
		else{
			only_call_function_if_this_tab_is_active(sync_with_google_calendar_helper);
		}
	}
	
	function sync_with_google_calendar_helper(){
		if (!check_if_guser_can_be_found_on_this_page()){
			return;
		}
		
		if (hasNoCache)
		{
			rebuild_local_event_cache(function(){}); // if never synced, rebuild
		}
		else
		{
			if (cacheIsNoLongerFresh || timeZoneOutOfSync)
			{
				rebuild_local_event_cache(function(){}); // if cache is out of date, rebuild
			}
			else if (lastUpdatedDate == null || lastUpdatedDate < new Date().addMinutes(-1 * LOCAL_STORAGE_UPDATE_FREQUENCY_MINUTES))
			{
				if (get_google_apps_sync_count_within_a_day() < 2){	// only update if we don't run into the multiple google apps selection screen bug
					setTimeout(function(){
						update_local_event_cache_silently(function(){});
					}, 45000); // A timeout to protect "wake up from sleep" errors.
				}
			}
		}
	}
}

function rebuild_local_event_cache(callback)
{
	var guser = get_gmail_user();

    var removeTheseIndexes = [];
    for (var i=0; i < localStorage.length; i++)
    {
        if (localStorage.key(i).indexOf("bcal") === 0 && localStorage.key(i).indexOf("settings") < 0 && localStorage.key(i).indexOf(guser) >= 0)
        {
            removeTheseIndexes.push(localStorage.key(i));
        }
    }
    for (var i=0; i < removeTheseIndexes.length; i++)
    {
        localStorage.removeItem(removeTheseIndexes[i]);
    }

	var minimumDate = new Date().addDays(DEFAULT_GET_EVENTS_MIN);
	var maximumDate = new Date().addDays(DEFAULT_GET_EVENTS_MAX);
	var currentTimeZone = get_olson_database_timezone_name();

    var url = "https://boomerangcalendar.baydin.com/geteventsbetween?guser=" + get_gmail_user() +
			"&minimumDate=" + minimumDate.valueOf() + "&maximumDate=" + maximumDate.valueOf() +
			"&timeZone=" + currentTimeZone;
    send_get_request_auth_wrapper(url, function(data){
		write_to_localStorage(LOCAL_STORAGE_LAST_UPDATED, new Date().toString("ddd, dd MMM yyyy HH:mm:ss"));
		write_to_localStorage(LOCAL_STORAGE_CACHE_MAXIMUM_DATE_VALID, maximumDate.toString("ddd, dd MMM yyyy HH:mm:ss"));
		write_to_localStorage(LOCAL_STORAGE_TIME_ZONE, get_olson_database_timezone_name());
		rebuild_local_event_cache_with_data(data);
		sync_preferences();
		if(callback!=null){
			callback();
		}
	});
}

function update_local_event_cache(callback){
	update_local_event_cache_helper(callback, false);
}

function update_local_event_cache_silently(callback){
	update_local_event_cache_helper(callback, true);
}

function update_local_event_cache_helper(callback, shouldBeSilent){
	if (!check_if_guser_can_be_found_on_this_page()){
		return;
	}
	
	var lastUpdatedDateStr = read_from_localStorage(LOCAL_STORAGE_LAST_UPDATED);
	
	if (lastUpdatedDateStr != null)
	{
		var lastUpdatedDate = Date.parse(lastUpdatedDateStr);
		var currentTimeZone = get_olson_database_timezone_name();

		write_to_localStorage(LOCAL_STORAGE_LAST_UPDATED, new Date().toString("ddd, dd MMM yyyy HH:mm:ss"));
		write_to_localStorage(LOCAL_STORAGE_TIME_ZONE, get_olson_database_timezone_name());
		var url = "https://boomerangcalendar.baydin.com/geteventsmodifiedsince?guser=" + get_gmail_user() +
			"&modifiedSinceDate=" + lastUpdatedDate.valueOf() + "&timeZone=" + currentTimeZone;
			
		var update_local_cache_after_auth = function(data){
			update_local_event_cache_with_data(data);
			if (callback!=null){
				callback();
			}
		};
			
		if (shouldBeSilent){
			send_get_request_silent_auth_wrapper(url, update_local_cache_after_auth);
		}
		else{
			send_get_request_auth_wrapper(url, update_local_cache_after_auth);
		}
	
	}
	else
	{
		rebuild_local_event_cache(callback);
	}
}

function full_calendar_sync_and_render_current_view(fullCalendarView, b4c_calendar){
	var viewStart = fullCalendarView.start;
	var viewEnd = fullCalendarView.end;
	var maximumCacheDateValidStr = read_from_localStorage(LOCAL_STORAGE_CACHE_MAXIMUM_DATE_VALID);
	var maximumCacheDateValid = Date.parse(maximumCacheDateValidStr);
	var minimumCacheDateValid = new Date().addDays(DEFAULT_GET_EVENTS_MIN);
	var currentTimeZone = get_olson_database_timezone_name();
		
	if(viewStart >= minimumCacheDateValid && viewEnd <= maximumCacheDateValid){
		//console.log("within cache");
	}
	else{
		var url = "https://boomerangcalendar.baydin.com/geteventsbetween?guser=" + get_gmail_user() +
		"&minimumDate=" + viewStart.valueOf() + "&maximumDate=" + viewEnd.valueOf()  +
		"&timeZone=" + currentTimeZone;
		
		send_get_request_silent_auth_wrapper(url, function(data){
			update_local_event_cache_with_data(data);
			rerender_b4c_calendar_after_cache_verification(b4c_calendar);
		});
	}

}

function verify_and_rerender_calendar_if_necessary($calendar){
	verify_local_event_cache(new Date(), function(){
		rerender_b4c_calendar_after_cache_verification($calendar);
	});
}

// verifies cache by checking entries around a particular date.
// if cache is invalid, then we rebuild.
// we run the callback function if the cache had to be rebuilt
function verify_local_event_cache(dateToCheck, callbackIfCacheHadToBeRebuilt){
	var minimumDate = dateToCheck.clone().addDays(-30).valueOf();
	var maximumDate = dateToCheck.clone().addDays(30).valueOf();
	var currentTimeZone = get_olson_database_timezone_name();

    var url = "https://boomerangcalendar.baydin.com/geteventsbetween?guser=" + get_gmail_user() +
			"&minimumDate=" + minimumDate + "&maximumDate=" + maximumDate  +
			"&timeZone=" + currentTimeZone;
    send_get_request_auth_wrapper(url, function(data){
		var cacheIsFresh = verify_local_event_cache_with_data(data);
		if (!cacheIsFresh){
			rebuild_local_event_cache(callbackIfCacheHadToBeRebuilt);
		}
	});
}

function verify_local_event_cache_with_data(data){
    var eventString = decodeURIComponent(data);
    var eventData = JSON.parse(eventString);

	var localStorageLookupStr = read_from_localStorage(LOCAL_STORAGE_LOOKUP);
	var localStorageEventsStr = read_from_localStorage(LOCAL_STORAGE_EVENTS);
	
	if (localStorageLookupStr == null || localStorageEventsStr == null){
		return false;
	}
	
	var localStorageLookup = JSON.parse(localStorageLookupStr);
	var localStorageEvents = JSON.parse(localStorageEventsStr);
	
	if (localStorageLookup == null || localStorageEvents == null){
		return false;
	}
	
	var i;
	var cacheIsFresh = true;
	for (i in eventData){
		var isInCache = verify_event_is_in_cache(eventData[i]["id"], Date.parse(eventData[i]["start"]), localStorageLookup, localStorageEvents);
		if (!isInCache){
			cacheIsFresh = false;
			break;
		}
	}
	return cacheIsFresh;
}


function verify_event_is_in_cache(eventId, eventStartDate, localStorageLookup, localStorageEvents)
{
	var foundInLookup = false;
	var foundInStorage = false;
	var dStr = eventStartDate.toString("yyyy-MM-dd");
	
	var lookup = localStorageLookup[dStr];
	var i;
	for (i in lookup){
		if (lookup[i] === eventId){
			foundInLookup = true;
			break;
		}
	}
	
	foundInStorage = eventId in localStorageEvents;
	
	return foundInLookup && foundInStorage;
}

function rebuild_local_event_cache_with_data(data)
{
    var eventString = decodeURIComponent(data);
    var eventData = JSON.parse(eventString);
    var eventStorage = {};
    var eventLookup = {};
	
	var i;
    for (i in eventData)
    {
        var d = Date.parse(eventData[i]["start"]);
		var dStr = d.toString("yyyy-MM-dd");
		append_or_create_entry(eventData[i]["id"], eventData[i], eventStorage);
		append_or_create_entry(dStr, eventData[i]["id"], eventLookup);
    }
    write_to_localStorage(LOCAL_STORAGE_EVENTS, JSON.stringify(eventStorage));
    write_to_localStorage(LOCAL_STORAGE_LOOKUP, JSON.stringify(eventLookup));
}

function update_local_event_cache_with_data(data)
{
    var eventString = decodeURIComponent(data);
    var eventData = JSON.parse(eventString);
    var eventStorage = JSON.parse(read_from_localStorage(LOCAL_STORAGE_EVENTS));
    var eventLookup = JSON.parse(read_from_localStorage(LOCAL_STORAGE_LOOKUP));

	var i;
    for (i in eventData)
    {
		// Delete existing entries, if they exist
		if (eventData[i]["id"] in eventStorage){
			var cancelledEvent = eventStorage[eventData[i]["id"]][0];
			var d = Date.parse(cancelledEvent["start"]);
			var dStr = d.toString("yyyy-MM-dd");

			delete eventStorage[eventData[i]["id"]];
			if (dStr in eventLookup){
				eventLookup[dStr] = eventLookup[dStr].filter(function(id){ return id != eventData[i]["id"]; });
			}
		}

		// Add new entries
		if (eventData[i]["status"] !== "cancelled"){
			var d = Date.parse(eventData[i]["start"]);
			var dStr = d.toString("yyyy-MM-dd");
			append_or_create_entry(eventData[i]["id"], eventData[i], eventStorage);
			append_or_create_entry(dStr, eventData[i]["id"], eventLookup);
		}
    }
	
    write_to_localStorage(LOCAL_STORAGE_EVENTS, JSON.stringify(eventStorage));
    write_to_localStorage(LOCAL_STORAGE_LOOKUP, JSON.stringify(eventLookup));
}

function get_cached_events_by_day(targetDate)
{

    var dStr = targetDate.toString("yyyy-MM-dd");
	
	
    var ids = jQuery.parseJSON(read_from_localStorage(LOCAL_STORAGE_LOOKUP))[dStr];
    var eventStorage = jQuery.parseJSON(read_from_localStorage(LOCAL_STORAGE_EVENTS));
	
    var events = [];
    for(entry in ids)
    {
       events.push(eventStorage[ids[entry]]);
    }

	
    return events;
}

function get_cached_tentative_events(newEventStart, newEventEnd, allDay)
{
	if (allDay){
		newEventStart = newEventStart.clone().set({hour:0, minute:0, second:0});
		newEventEnd = newEventEnd.clone().set({hour:23, minute:59, second:0});
	}
	var conflictingTentativeEvents = [];
	var localStorageLookupStr = read_from_localStorage(LOCAL_STORAGE_LOOKUP);
	var localStorageEventsStr = read_from_localStorage(LOCAL_STORAGE_EVENTS);
	if (localStorageLookupStr == null || localStorageEventsStr == null){
		return conflictingTentativeEvents;
	}
	var localStorageLookupJson = jQuery.parseJSON(localStorageLookupStr);
	var dStr = newEventStart.toString("yyyy-MM-dd");
	var ids = localStorageLookupJson[dStr];
	var dStr2 = newEventEnd.toString("yyyy-MM-dd");
	// if multiday event, check both dates
	if (ids != null && dStr != dStr2)
	{
		ids.push(localStorageLookupJson[dStr2]);
	}
	var eventStorage = jQuery.parseJSON(localStorageEventsStr);
	var entry;

	for(entry in ids) {
		var existingEvent = eventStorage[ids[entry]];
		if (existingEvent)
		{
			var eventTitle = existingEvent[0]["title"];
			var existingEventStart = Date.parse(existingEvent[0]["start"]);
			var existingEventEnd = Date.parse(existingEvent[0]["end"]);
			if (newEventStart < existingEventEnd && existingEventStart < newEventEnd)
			{
					if (eventTitle.indexOf("[?]") == 0 && existingEvent[0]["tentativeSetId"] != null){
						conflictingTentativeEvents.push(existingEvent);
					}
					
			}
		}
	}
	return conflictingTentativeEvents;
}

function get_cached_free_busy(newEventStart, newEventEnd, allDay)
{
	if (allDay){
		newEventStart = newEventStart.clone().set({hour:0, minute:0, second:0});
		newEventEnd = newEventEnd.clone().set({hour:23, minute:59, second:0});
	}

	var localStorageLookupStr = read_from_localStorage(LOCAL_STORAGE_LOOKUP);
	var localStorageEventsStr = read_from_localStorage(LOCAL_STORAGE_EVENTS);
	
	// in the rare case that we haven't sync'd yet, just return available
	if (localStorageLookupStr == null || localStorageEventsStr == null){
		return "b4c-black";
	}
	
	var localStorageLookupJson = JSON.parse(localStorageLookupStr);

    var dStr = newEventStart.toString("yyyy-MM-dd");

    var ids = localStorageLookupJson[dStr];
	
    var dStr2 = newEventEnd.toString("yyyy-MM-dd");
    if (ids != null && dStr !== dStr2)
	{
		var idOfEndTime = localStorageLookupJson[dStr2];
		if (idOfEndTime != null){
			ids.concat(localStorageLookupJson[dStr2]);
		}
    }
		
    var eventStorage = JSON.parse(localStorageEventsStr);
    var events = [];
    var conflict = "b4c-green";
	var entry;
	
    for(entry in ids)
    {
        var thisEvent = eventStorage[ids[entry]];
	
		var eventTitle = thisEvent[0]["title"];
		
        var existingEventStart = Date.parse(thisEvent[0]["start"]);
        var existingEventEnd = Date.parse(thisEvent[0]["end"]);
		
		if (thisEvent[0]["allDay"] && conflict !== "b4c-red"){
			conflict = "b4c-yellow"; // all day or single event overlap with all day event
		}
        else if (newEventStart < existingEventEnd && existingEventStart < newEventEnd)
        {
			if (allDay){
				conflict = "b4c-yellow"; // all day overlap with single event
			}
			else{
				if (eventTitle.indexOf("[?]") === 0){
					conflict = "b4c-yellow"; // single event overlap with tentative event
				}
				else{
					conflict = "b4c-red"; // single event overlap with single event
				}
			}
        }
        else if (newEventStart < existingEventEnd.add(1).hours() &&
            existingEventStart.add(-1).hours() < newEventEnd && conflict !== "b4c-red")
        {
           conflict = "b4c-yellow"; // single adjacent to single event
        }
    }
    return conflict;
}

function append_or_create_entry(key, value, dataObject)
{
	if (key in dataObject)
	{
		if(!(value in dataObject[key]))
		{
			dataObject[key].push(value);
		}
	}
	else
	{
		dataObject[key] = [value];
	}
}


////////////// Local Storage Utility Functions //////////////////
function write_to_localStorage(key, value)
{
	var guser = get_gmail_user();
	key += guser;

    if(typeof(localStorage) == undefined)
    {
        alert("Your browser does not support HTML5 localStorage. How the heck are you using Boomerang Calendar?");
    }
    else
    {
        try
        {
            if(typeof(key) != undefined && key != null)
            {
                localStorage.setItem(key, value);
            }
        }
        catch (e)
        {
            if (e == QUOTA_EXCEEDED_ERR)
            {
                alert("HTML5 Storage Quota exceeded! Please let us know this happened at support@baydin.com");
            }
            else
            {
                alert("Something went wrong writing to HTML5 LocalStorage. Please let us know at support@baydin.com");
            }
        }
    }
}

function read_from_localStorage(key)
{
	var guser = get_gmail_user();
	key += guser;

    if(typeof(localStorage) == undefined)
    {
        return null;
    }
    else
    {
        try
        {
            return localStorage.getItem(key);
        }
        catch (e)
        {
            return null;
        }
    }
}

function get_all_events_from_local_storage()
{
	var events = [];
	var localStorageEventsStr = read_from_localStorage(LOCAL_STORAGE_EVENTS);
	if (localStorageEventsStr != null){
		var eventData = jQuery.parseJSON(localStorageEventsStr);
		var i;
		for (i in eventData)
		{
			events.push(eventData[i][0]);
		}
	}
	return events;
}

function get_last_updated_time_for_ui()
{
	var lastUpdatedTime = read_from_localStorage(LOCAL_STORAGE_LAST_UPDATED);
	if (lastUpdatedTime == null || Date.parse(lastUpdatedTime) == null){
		lastUpdatedTime = "now";
	}
	else{
		var msDifference = new Date().valueOf() - Date.parse(lastUpdatedTime).valueOf();
		var minutesDifference = Math.floor(msDifference/1000/60);
		if (minutesDifference < 60){
			lastUpdatedTime = minutesDifference + " minutes ago.";
		}
		else{
			lastUpdatedTime = " over an hour ago.";
		}
	}
	return lastUpdatedTime;
}


function check_if_guser_can_be_found_on_this_page(){
	if (!get_gmail_user()){
		show_dialog("We are unable to identify your gmail address." +
					" To help us improve our beta product, please email us at support@baydin.com" +
					" with details of how you encountered this error.");
		return false;
	}
	else{
		return true;
	}
}

function sync_preferences(optional_callback){
    var url = "https://boomerangcalendar.baydin.com/getpreferences?guser=" + get_gmail_user();
    send_get_request_silent_auth_wrapper(url, function(data){
    	write_to_localStorage(LOCAL_STORAGE_PREFERENCES, data);
    	if(optional_callback){
    		optional_callback();	
    	}
	});
}

function get_preferences(){
	var defaultPreferences = {"startOfWeek": 0, "defaultMeetingTime": 30,
							"customDateFormat": "dddd, MMM d", "customTimeFormat": "h:mm tt",
							"includeTimeZone": false};
	try{
		var preferences = read_from_localStorage(LOCAL_STORAGE_PREFERENCES);
		var preferencesData = jQuery.parseJSON(preferences);
		return preferencesData || defaultPreferences;
	}
	catch(err){
		return defaultPreferences;
	}
}



// LOCATIONS

var LOCAL_STORAGE_SAVED_LOCATIONS = "saved_locations";

function add_location_to_local_storage(displayText, type, text, html){
    type = type || "place";
    text = text || displayText;
    html = html || text;

    var locations = get_saved_locations_from_local_storage();
    var alreadyExists = false;
    for (var index in locations){
        if (text == locations[index]["text"]){
            alreadyExists = true;
            break;
        }
    }
    if (!alreadyExists){
        locations.push({
            "displayText": displayText,
            "text": text,
            "html": html,
            "date": new Date().valueOf(),
            "type": type,
            "uid": Math.random().toString(36).substr(2,9)
        })
        save_locations_in_local_storage(locations);
    }
}

function remove_location_from_local_storage(uid){
    var locations = get_saved_locations_from_local_storage();
    locations = locations.filter(function(location){
        return location["uid"] != uid;
    });
    save_locations_in_local_storage(locations);
}

function update_location_in_local_storage(uid){
    var locations = get_saved_locations_from_local_storage();
    for (var index in locations){
        if (locations[index]["uid"] == uid){
            locations[index]["date"] = new Date().valueOf();
        }
    }
    save_locations_in_local_storage(locations);
}

function save_locations_in_local_storage(locations){
    var locationString = JSON.stringify(locations);
    return write_to_localStorage(LOCAL_STORAGE_SAVED_LOCATIONS, locationString) || [];
}

function get_sorted_locations_from_local_storage(){
    var locations = sort_locations(get_saved_locations_from_local_storage());
    locations = filter_out_top_five_places_from_sorted_locations(locations);
    return locations;
}

function get_saved_locations_from_local_storage(){
    return JSON.parse(read_from_localStorage(LOCAL_STORAGE_SAVED_LOCATIONS)) || [];
}

function sort_locations(locations){
    function sort_locations_for_dropdown(a, b){
        var typeArray = ["place", "phone", "online"];
        if (a["type"] !=  b["type"]){
            return typeArray.indexOf(a["type"]) - typeArray.indexOf(b["type"]);
        }
        else{
            return b["date"] - a["date"];
        }
    }

    locations.sort(sort_locations_for_dropdown);
    return locations;
}

function filter_out_top_five_places_from_sorted_locations(locations){
	var placeCount = 0;
	for (var i in locations){
	  if (locations[i]["type"] == "place"){
	    placeCount++;
	  }
	}
	if (placeCount > 5){

	  locations.splice(5, placeCount - 5)
	}
	return locations;
}